/*******************************************************************************
 *  Copyright 2007 SWTBot, http://swtbot.org/
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 ******************************************************************************/
package net.sf.swtbot.eclipse.finder.finder;

import java.util.ArrayList;
import java.util.List;

import net.sf.swtbot.eclipse.finder.widgets.SWTBotCommand;
import net.sf.swtbot.finder.UIThreadRunnable;
import net.sf.swtbot.finder.UIThreadRunnable.ListResult;
import net.sf.swtbot.matcher.IMatcher;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.Command;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;

/**
 * Finds all the contribution items within the application.
 *
 * @author @author Stephen Paulin &lt;paulin [at] spextreme [dot] com&gt;
 * @version $Id$
 * @since 1.2
 */
public class CommandFinder {
	/**
	 * The logging instance for this class.
	 */
	private static final Logger	log	= Logger.getLogger(CommandFinder.class);

	/**
	 * Creates a CommandFinder.
	 */
	public CommandFinder() {
		// Do nothing.
	}

	/**
	 * Finds a command matching the given item.
	 *
	 * @param matcher the matcher that can match commands.
	 * @return all command that match the matcher.
	 */
	public List findCommand(IMatcher matcher) {
		return findCommand(getCommandService(), matcher);
	}

	/**
	 * Gets a list of all command matching the matcher.
	 *
	 * @param service the {@link ICommandService} instance to use.
	 * @param matcher the matcher that can match the command item.
	 * @return The list of {@link Command}s that match the matcher.
	 */
	public List findCommand(final ICommandService service, final IMatcher matcher) {
		return UIThreadRunnable.syncExec(new ListResult() {

			public List run() {
				List l = new ArrayList();
				Command[] commands = service.getDefinedCommands();

				for (int i = 0; i < commands.length; i++)
					try {
						String name = commands[i].getName();
						if (matcher.match(name))
							l.add(new SWTBotCommand(commands[i]));
					} catch (Exception e) {
						log.error("Failed with an exception on the command: " + commands[i].toString(), e);
					}

				return l;
			}
		});
	}

	/**
	 * Gets the command service.
	 *
	 * @return The {@link ICommandService}.
	 */
	protected ICommandService getCommandService() {
		return (ICommandService) PlatformUI.getWorkbench().getService(ICommandService.class);
	}
}
